import os
import json


from .download import Download
from .clean import Clean
from ..utils import BASE_PATH


DRIVER = BASE_PATH / "executable"

download = Download()
clean = Clean()


def get_installed_chrome_path():
    config_file = DRIVER / "chrome.json"
    if os.path.isfile(config_file):
        chromedriver = json.load(open(config_file, "r"))["chromedriver"]
        if chromedriver:
            return chromedriver

    chrome_version = download.check_installed_chrome_version()
    if chrome_version:
        chrome_driver_name = download.get_chrome_driver_download_link(chrome_version)
        download.download_chrome_driver(chrome_driver_name)
        filename = download.extract_chrome_driver_zip(chrome_driver_name)
        path = filename
        os.chmod(path, 755)
        clean.remove_signature_in_javascript(path)
        return path


def check_user_agent(driver):
    original_user_agent_string = driver.execute_script("return navigator.userAgent")
    driver.execute_cdp_cmd(
        "Network.setUserAgentOverride",
        {
            "userAgent": original_user_agent_string.replace("Headless", ""),
        },
    )
    return driver


def check_navigator(driver):
    if driver.execute_script("return navigator.webdriver"):
        driver.execute_cdp_cmd(
            "Page.addScriptToEvaluateOnNewDocument",
            {
                "source": """

                            Object.defineProperty(window, 'navigator', {
                                value: new Proxy(navigator, {
                                has: (target, key) => (key === 'webdriver' ? false : key in target),
                                get: (target, key) =>
                                    key === 'webdriver'
                                    ? undefined
                                    : typeof target[key] === 'function'
                                    ? target[key].bind(target)
                                    : target[key]
                                })
                            });
                """
            },
        )
    return driver



from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.common.exceptions import WebDriverException
import random
import os
from fake_useragent import UserAgent
import sys

sys.path.append("......")
# Local Imports
from .clean import Clean
from ..utils import Util
from ..constants import USER_AGENTS, SYSTEM
from .download import Download
from .check import check_user_agent, check_navigator
from easy_selenium.logger import logger

clean = Clean()
util = Util()
download = Download()


class Driver:
    """
    Create a new undetectable Chrome driver instance.
    """

    def __init__(self, user_agent=None):
        self.user_agent = user_agent
        self.generate_user_agent()

    def generate_user_agent(self):
        try:
            ua = UserAgent()
            user_agent = ua["google chrome"]
        except IndexError:
            user_agent = random.choice(USER_AGENTS)
        self.user_agent = user_agent

    def create(
        self,
        headless=False,
        profile_path="",
        mute=False,
    ):
        from .check import get_installed_chrome_path

        path = get_installed_chrome_path()

        if path is not None:
            options = webdriver.ChromeOptions()
            options.headless = headless
            options.add_argument("--start-maximized")
            options.add_argument("--lang=en-US")
            if profile_path:
                options.add_argument(r"--user-data-dir=%s" % profile_path)
            options.add_experimental_option(
                "excludeSwitches", ["enable-automation", "enable-logging"]
            )
            options.add_argument("--disable-blink-features=AutomationControlled")
            options.add_argument("--log-level=3")
            options.add_experimental_option("useAutomationExtension", False)
            options.add_argument(f"user-agent={self.user_agent}")
            if mute:
                options.add_argument("--mute-audio")
                options.add_argument("--no-sandbox")
            try:
                s = Service(path)
                driver = webdriver.Chrome(service=s, options=options)
            except WebDriverException:
                driver = webdriver.Chrome(executable_path=path, options=options)
            except Exception as error:
                logger.critical(f"[-] Driver:Create {error}")
            else:
                driver = check_navigator(driver)
                driver = check_user_agent(driver)
                driver.get("https://selmi.tech")
                return driver
        else:
            logger.error("Make sure you have chrome installed on your machine!")


class Remote:
    def create(self, profile=None, debug_port=9222, control_existing_instance=False):
        options = webdriver.ChromeOptions()
        from .check import get_installed_chrome_path

        path = get_installed_chrome_path()
        if profile:
            options.add_argument(r"--user-data-dir=%s" % profile)
        if not control_existing_instance:
            self.open_new_cmd_and_run_command(
                profile_path=profile,
                port=debug_port,
            )
        options.add_experimental_option("debuggerAddress", f"127.0.0.1:{debug_port}")

        try:
            driver = webdriver.Chrome(service=Service(path), options=options)
        except TypeError:
            driver = webdriver.Chrome(executable_path=path, options=options)
        except Exception as error:
            logger.critical(f"[-] Driver:Create {error}")
        else:
            driver.get("https://selmi.tech")
            return driver

    def open_new_cmd_and_run_command(self, profile_path="", port=9222):
        """
        This function opens a new cmd window and runs the command
        a command that runs the Chrome driver in the debugging mode.
        It does not work for Firefox
        """

        if profile_path:
            cmd = f'chrome.exe --remote-debugging-port={port} --user-data-dir="{profile_path}"'
            if SYSTEM == "Windows":
                profile_path = (
                    profile_path.replace("/", "\\")
                    if "/" in profile_path
                    else profile_path
                )
        else:
            cmd = f"chrome.exe -remote-debugging-port={port}"
        if SYSTEM == "Windows":
            os.system("start cmd /k " + cmd)
        else:
            google = util.whereis_google_chrome()
            if not google:
                raise Exception("whereis: Google Chrome is not installed")
            if not profile_path:
                cmd = f"{google} --remote-debugging-port={port}"
            else:
                cmd = f'{google} --remote-debugging-port={port} --user-data-dir="{profile_path}"'
            os.system(f'gnome-terminal -- bash -c "{cmd}"')



from termcolor import colored
from colorama import init
import os
import platform
import subprocess


from ..utils import BASE_PATH


DRIVER = BASE_PATH / "executable"
init()


class Clean:
    def update_binary(self, path):
        word = "cdc_".encode()
        new = "tch_".encode()
        path = path
        while True:
            string = b""
            Flag = 0
            with open(path, "r+b") as file:
                pos = 0
                data = string = file.read(1)
                while data:
                    data = file.read(1)
                    if data == b" ":
                        if word in string:
                            new_tring = string.decode().replace(
                                word.decode(), new.decode()
                            )
                            file.seek(pos)
                            file.write(new_tring.encode())
                            Flag = 1
                            break
                        else:
                            pos = file.tell()
                            data = string = file.read(1)
                    else:
                        string += data
                        continue
            if not Flag:
                break

    def remove_signature_in_javascript(self, chromedriver):
        try:
            if platform.system() == "Windows":
                os.chmod(chromedriver, 755)
            else:
                os.chmod(chromedriver, 777)
                subprocess.Popen(f"sudo chmod 777 {chromedriver}", stdout=subprocess.PIPE, shell=True)
            with open(chromedriver, "r", errors="ignore") as chrome:
                content = chrome.read()
            content = content.replace("cdc_", "tch_")
            self.update_binary(chromedriver)
        except Exception as error:
            print(colored("[-] Signature: ", "red") + str(error))



from loguru import logger
from decouple import config, UndefinedValueError

try:
    debug = config("BACKTRACE")
except UndefinedValueError:
    debug = 0
logger.add(
    "debug.log",
    format="{time: YYYY-MM-DD on HH:MM} | {level} | {message}",
    level="INFO",
    rotation="10 MB",
    retention="10 days",
    backtrace=True if debug else False,
    diagnose=True if debug else False,
)



import platform


USER_AGENTS = [
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36",
]
SYSTEM = platform.system()



import requests
import platform
import io
import zipfile
from bs4 import BeautifulSoup
from termcolor import colored
from colorama import init


from ..utils import Util, BASE_PATH

OSNAME = platform.system()
init()
util = Util()
DRIVER = BASE_PATH / "executable"


class Download:
    """
    It will get the installed Chrome driver and based on the operating system
    it will download the compatible chromedriver.
    """

    def get_gecko_driver_latest_link(self):
        base_url = "https://github.com/mozilla/geckodriver/releases/latest"
        headers = {
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36"
        }
        response = requests.get(base_url, headers=headers)
        content = response.content.decode("utf-8")
        soup = BeautifulSoup(content, features="lxml")
        urls = [
            link.get("href")
            for link in soup.find_all("a")
            if "geckodriver-v" in link.get("href")
        ]
        try:
            if OSNAME == "Windows":
                return (
                    "https://github.com"
                    + [url for url in urls if "win32.zip" in url][0]
                )
            elif OSNAME == "Linux":
                return (
                    "https://github.com"
                    + [url for url in urls if "linux32.tar.gz" in url][0]
                )
            else:
                return (
                    "https://github.com"
                    + [url for url in urls if "macos.tar.gz" in url][0]
                )
        except IndexError:
            return None

    def download_gecko_driver(self):
        url = self.get_gecko_driver_latest_link()
        headers = {
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36"
        }
        try:
            response = requests.get(url, stream=True, headers=headers)
            return response
        except Exception as error:
            print(colored("Geckodriver:Download - ", "red") + str(error))
            return None

    def extract_gecko_driver_zip(self):
        response = self.download_gecko_driver()
        if response:
            z = zipfile.ZipFile(io.BytesIO(response.content))
            z.extractall(DRIVER)
            print(colored("[+]", "green") + " Gecko driver has been installed.")
            geckodriver_path = DRIVER / "geckodriver"
            geckodriver_path = (
                str(geckodriver_path) + ".exe" if OSNAME == "Windows" else ""
            )
            util.update_gecko_driver_config(geckodriver=geckodriver_path)
            return geckodriver_path
        return None



import json
import os
import platform
from pathlib import Path
import subprocess

from loguru import logger

BASE_PATH = Path(__file__).resolve().absolute().parent.parent
DRIVER = BASE_PATH / "executable"


class Util:
    def create_chrome_driver_config(self):
        self.create_folder_if_not_exist(DRIVER)
        config_file = DRIVER / "chrome.json"
        if not os.path.isfile(config_file):
            with open(config_file, "w") as config:
                data = {"system": platform.system(), "version": "", "chromedriver": ""}
                config.write(json.dumps(data))
        return config_file

    def create_gecko_driver_config(self):
        self.create_folder_if_not_exist(DRIVER)
        config_file = DRIVER / "gecko.json"
        if not os.path.isfile(config_file):
            with open(config_file, "w") as config:
                data = {
                    "system": platform.system(),
                    "version": "",
                    "geckodriver": "",
                    "binary": "",
                }
                config.write(json.dumps(data))
        return config_file

    def update_chrome_driver_config(self, version="", chromedriver=""):
        config_file = self.create_chrome_driver_config()
        config_data = json.load(open(config_file, "r"))
        if version:
            config_data["version"] = version
        if chromedriver:
            config_data["chromedriver"] = chromedriver
        with open(config_file, "w") as config:
            config.write(json.dumps(config_data))

    def update_gecko_driver_config(self, version="", geckodriver="", binary=""):
        config_file = self.create_gecko_driver_config()
        config_data = json.load(open(config_file, "r"))
        if version:
            config_data["version"] = version
        if binary:
            config_data["binary"] = binary
        if geckodriver:
            config_data["geckodriver"] = geckodriver
        with open(config_file, "w") as config:
            config.write(json.dumps(config_data))

    def whereis_google_chrome(self):
        """
        Get the path of the chrome driver
        """
        try:
            chrome_path = (
                subprocess.check_output(["whereis", "google-chrome"])
                .decode("utf-8")
                .strip()
                .split()[1]
            )
        except Exception as error:
            logger.error(error)
            return False
        return chrome_path

    def create_folder_if_not_exist(self, path):
        if not os.path.isdir(path):
            os.mkdir(path)
            return True
        return True



import os
import json


from ..utils import BASE_PATH, Util


DRIVER = BASE_PATH / "executable"
util = Util()


def get_installed_gecko_path():
    config_file = os.path.join(DRIVER, "gecko.json")
    if os.path.isfile(config_file):
        geckodriver = json.load(open(config_file, "r"))["geckodriver"]
        if geckodriver:
            return geckodriver

    return None


def get_firefox_binary_path(new_path=None):
    config_file = util.create_gecko_driver_config()
    if os.path.isfile(config_file):
        config = json.load(open(config_file, "r"))
        binary = json.load(open(config_file, "r"))["binary"]
        if binary:
            return binary
        elif new_path:
            path = new_path.replace("\\", "/")
            with open(DRIVER / "gecko.json", "w") as cfg:
                config["binary"] = path
                cfg.write(json.dumps(config))
                return path
    return None



import time
import pickle
import random
import os
from ..logger import logger
from ..driver.utils import Util


util = Util()
BASE_PATH = os.getcwd()


class Cookies:
    """
    Save and load cookies.
    """
    def save(self, driver, url, cookies_file_name, cookies_folder_path=None):
        """
        Save cookies with .pkl extension.

        Args:
            driver (WebDriver): A chrome or Gecko driver.
            url (str): The url of the website.
            cookies_file_name (str): The name of the cookies file.
            cookies_folder_path (str): absolute path of the cookies folder
        """
        if not cookies_file_name.endswith(".pkl"):
            cookies_file_name += ".pkl"
        if cookies_folder_path:
            util.create_folder_if_not_exist(cookies_folder_path)
            path = os.path.join(cookies_folder_path, cookies_file_name)
        else:
            cookies_folder = os.path.join(BASE_PATH, "cookies")
            util.create_folder_if_not_exist(cookies_folder)
            path = os.path.join(cookies_folder, cookies_file_name)
        driver.get(url)
        time.sleep(random.randint(1, 10))
        pickle.dump(driver.get_cookies(), open(path, "wb"))
        logger.success(f"[+] {cookies_file_name} has been saved.")

    def load(self, driver, url, cookies_file_name, cookies_folder_path=None):
        """
        Load cookies from the cookies folder.

        Args:
            driver (WebDriver): A chrome or Gecko driver.
            url (str): The url of the website.
            cookies_file_name (str): The name of the cookies file.
            cookies_folder_path (str): absolute path of the cookies folder
        """
        if not cookies_file_name.endswith(".pkl"):
            cookies_file_name += ".pkl"
        if cookies_folder_path:
            util.create_folder_if_not_exist(cookies_folder_path)
            path = os.path.join(cookies_folder_path, cookies_file_name)
        else:
            cookies_folder = os.path.join(BASE_PATH, "cookies")
            util.create_folder_if_not_exist(cookies_folder)
            path = os.path.join(cookies_folder, cookies_file_name)
        driver.get(url)
        cookies = pickle.load(open(path, "rb"))
        for cookie in cookies:
            driver.add_cookie(cookie)
        logger.success(f"[+] {cookies_file_name} has been loaded.")



from selenium import webdriver
from selenium.webdriver.firefox.service import Service
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import random
from fake_useragent import UserAgent
import sys

# Local Imports
from ..utils import Util
from ..constants import USER_AGENTS
from .download import Download
from easy_selenium.logger import logger
from .check import get_firefox_binary_path

sys.path.append("....")
download = Download()
util = Util()


class Driver:
    """
    Create a new driver instance (Chrome & Firefox).
    """

    def __init__(self, binary_path, user_agent=None):
        self.user_agent = user_agent
        self.generate_user_agent()
        self.binary_path = get_firefox_binary_path(binary_path)

    def generate_user_agent(self):
        try:
            ua = UserAgent()
            user_agent = ua["google chrome"]
        except IndexError:
            user_agent = random.choice(USER_AGENTS)
        self.user_agent = user_agent

    def create(self, headless=False, profile=None, mute=False):
        from ..firefox.check import get_installed_gecko_path

        path = get_installed_gecko_path()
        if not path:
            path = download.extract_gecko_driver_zip()
        if path is not None:
            options = webdriver.FirefoxOptions()
            options.headless = headless
            options.set_preference("dom.webnotifications.serviceworker.enabled", False)
            options.set_preference("dom.webnotifications.enabled", False)
            firefox_profile = webdriver.FirefoxProfile(profile)
            firefox_profile.set_preference("intl.accept_languages", "en-US")
            firefox_profile.set_preference(
                "general.useragent.override", self.user_agent
            )
            firefox_profile.set_preference("dom.webdriver.enabled", False)
            firefox_profile.set_preference("useAutomationExtension", False)
            firefox_profile.update_preferences()
            options.profile = firefox_profile
            desired = DesiredCapabilities.FIREFOX
            desired["marionette"] = True
            if mute:
                options.add_argument("--mute-audio")
                options.add_argument("--no-sandbox")
            try:
                s = Service(path)
                driver = webdriver.Firefox(
                    service=s,
                    firefox_binary=self.binary_path,
                    options=options,
                    desired_capabilities=desired,
                )
            except (TypeError, WebDriverException):
                driver = webdriver.Firefox(
                    executable_path=path,
                    firefox_binary=self.binary_path,
                    options=options,
                    desired_capabilities=desired,
                )
            except Exception as error:
                logger.critical(f"[-]  {error}")
            else:
                driver.maximize_window()
                driver.get("https://selmi.tech")
                return driver
        else:
            logger.error("Make sure you have Firefox installed on your machine!")



import time
from random import uniform
import sys


def send_keys_one_by_one(controller, keys, min_delay=0.05, max_delay=0.25):
    for key in keys:
        controller.send_keys(key)
        time.sleep(uniform(min_delay, max_delay))


def driver_safe_quit(driver, exit=True):
    try:
        driver.quit()
    except Exception as error:
        print(error)
        pass
    if exit:
        sys.exit()


def send_gmail(message, sender, password, receiver):
    import smtplib, ssl

    port = 587
    smtp_server = "smtp.gmail.com"
    sender_email = sender
    receiver_email = receiver
    password = password
    message = message
    context = ssl.create_default_context()
    with smtplib.SMTP(smtp_server, port) as server:
        server.ehlo()
        server.starttls(context=context)
        server.ehlo()
        server.login(sender_email, password)
        server.sendmail(sender_email, receiver_email, message)


def scoll_until_the_end(driver):
    SCROLL_PAUSE_TIME = 0.5
    last_height = driver.execute_script("return document.body.scrollHeight")
    while True:
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        time.sleep(SCROLL_PAUSE_TIME)
        new_height = driver.execute_script("return document.body.scrollHeight")
        if new_height == last_height:
            break
        last_height = new_height


def scroll_to(driver, element):
    driver.execute_script("window.scrollTo(0, arguments[0]);", element)


def get_user_agent(driver):
    return driver.execute_script("return navigator.userAgent")


def page_screenshot(driver, file_name):
    driver.get_screenshot_as_file(file_name)


def element_screenshot(element, file_name):
    element.screenshot(file_name)



import setuptools

__version__ = "0.1.4"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='easy py selenium',
    version=__version__,
    author='Selmi Abderrahim',
    author_email='contact@selmi.tech',
    description='A tool that makes working with selenium easier.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/SelmiAbderrahim/easy-selenium',
    project_urls = {
        "Bug Tracker": "https://github.com/SelmiAbderrahim/easy-selenium/issues"
    },
    include_package_data=True,
    package_data={
        "easy_selenium.executable": ["*.json"],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    keywords='python automation tools with selenium',
    license='MIT',
    packages=setuptools.find_packages(include=["easy_selenium", "easy_selenium.*"], exclude=["easy_selenium.tests", "easy_selenium.tests.*"]),
    install_requires=['lucd==0.1.7', 'beautifulsoup4==4.11.1', 'black==22.6.0', 'python-decouple==3.6', 'loguru==0.6.0'],
)



import random
import os
import time
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains
from selenium.common.exceptions import TimeoutException
from ...logger import logger
from ...driver.utils import Util
from ...cookies.cookies import Cookies
from ...common.funcs import send_keys_one_by_one

util = Util()
WORKING_DIRECTORY = os.getcwd()


class Login(Cookies):
    """
    Automate LinkedIn authentication.

    Args:
        email (str): The LinkedIn email address
        password (str): The LinkedIn password

    Returns:
        The return value. True for success, False otherwise
    """
    BASE_URL = "https://www.linkedin.com"

    def __init__(self, email, password):
        self.email = email
        self.password = password
        self.cookie_name = f"linkedin_{email}"
        self.cookie_folder = os.path.join(WORKING_DIRECTORY, "cookies")
        util.create_folder_if_not_exist(self.cookie_folder)

    def get_signin_link(self, driver):
        try:
            signin = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//a[contains(@class,'nav__button-secondary')]")
                )
            )
        except TimeoutException:
            signin = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located((By.XPATH, "//a[text()='Sign in']"))
            )
        try:
            return signin, None
        except Exception as error:
            return None, error

    def is_logged_in(self, driver):
        if (
            driver.current_url
            == "https://www.linkedin.com/error_pages/unsupported-browser.html"
        ):
            try:
                skip = WebDriverWait(driver, 5).until(
                    EC.presence_of_element_located(
                        (By.XPATH, "//a[@title= 'continue anyway']")
                    )
                )
            except TimeoutException:
                pass
            except Exception as error:
                logger.error(
                    """
                LinkedIn requires an updated browser to work, Please update yours:
                https://www.linkedin.com/error_pages/unsupported-browser.html
                """
                    + str(error)
                )
            else:
                ActionChains(driver).move_to_element(skip).click().perform()
        time.sleep(random.randint(1, 5))
        if "feed" not in driver.current_url:
            return False
        return True

    def credentials(self, driver):
        time.sleep(random.randint(1, 5))
        signin, error = self.get_signin_link(driver)
        if signin:
            ActionChains(driver).move_to_element(signin).click().perform()
        else:
            driver.get("https://www.linkedin.com/login")
        try:
            login_input = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located((By.ID, "session_key"))
            )
        except TimeoutException:
            login_input = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located((By.ID, "username"))
            )
        send_keys_one_by_one(login_input, self.email)
        time.sleep(random.randint(1, 5))
        try:
            password_input = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located((By.ID, "session_password"))
            )
        except TimeoutException:
            password_input = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located((By.ID, "password"))
            )
        send_keys_one_by_one(password_input, self.password)
        time.sleep(random.randint(1, 5))
        try:
            submit_button = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//button[@class= 'sign-in-form__submit-button']")
                )
            )
        except TimeoutException:
            submit_button = WebDriverWait(driver, 5).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//button[@aria-label= 'Sign in']")
                )
            )
        ActionChains(driver).move_to_element(submit_button).click().perform()
        time.sleep(random.randint(1, 3))
        if self.is_logged_in(driver):
            self.save(driver, self.BASE_URL, self.cookie_name)
            return driver, True
        return driver, False

    def welcome_back_password(self, driver):
        try:
            WebDriverWait(driver, 5).until(
                EC.presence_of_element_located((By.XPATH, "//a[text()='Sign in']"))
            )
        except TimeoutException:
            pass
        else:
            try:
                password_input = WebDriverWait(driver, 5).until(
                    EC.presence_of_element_located((By.ID, "password"))
                )
            except TimeoutException:
                password_input = WebDriverWait(driver, 5).until(
                    EC.presence_of_element_located(
                        (By.XPATH, "//input[@aria-label='Password']")
                    )
                )
            send_keys_one_by_one(password_input, self.password)

    def login_cookies(self, driver):
        cookie_file_path = os.path.join(self.cookie_folder, self.cookie_name) + ".pkl"
        if os.path.isfile(cookie_file_path):
            self.load(driver, self.BASE_URL, self.cookie_name)
            driver.get(self.BASE_URL)
            self.welcome_back_password(driver)
            if self.is_logged_in(driver):
                return driver, True
        return driver, False

    def start(self, driver):
        driver.get(self.BASE_URL)
        time.sleep(random.randint(1, 10))
        driver, success = self.login_cookies(driver)
        SUCCESS_C = f"[C] {self.email} logged in successfully!"
        SUCCESS_P = f"[P] {self.email} logged in successfully!"
        FAILURE = f"[!] {self.email} failed to log in!"
        if success:
            logger.success(SUCCESS_C)
            return True
        else:
            driver, success = self.credentials(driver)
            if success:
                logger.success(SUCCESS_P)
                return True
        logger.error(FAILURE)
        return False
